<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

require_once MOONY_ROOT_DIR . '/Moony/Helpers.php';

/**
 * アクションの基底クラスです。
 * 全てのアクションクラスはこのクラスを継承して実装してください。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Action
{
    /**
     * Moonyのインスタンス（動作設定）
     * @var object
     */
    var $moony;

    /**
     * Moony_Requestのインスタンス
     * @var object
     */
    var $request;

    /**
     * Moony_Sessionのインスタンス
     * @var object
     */
    var $session;

    /**
     * PATH_INFO経由で渡されたパラメータの配列
     * @var array
     */
    var $args;

    /**
     * アクションの処理を実行します。
     * 実装時にオーバーライドしてください。
     * オーバーライドしない場合、テンプレートの処理結果を出力します。
     *
     * @access public
     */
    function execute()
    {
        $this->render();
    }

    /**
     * テンプレートの処理結果を取得します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、falseを返します。
     * テンプレートからアクションクラスのpublicなメンバ変数には
     * <?php h($this->foo); ?>とアクセスすることができます。
     * $local_varsが設定されている場合、値は展開されて
     * <?php h($foo); ?>のように利用することができます。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @return string|bool テンプレートの処理結果
     */
    function fetch($template = null, $local_vars = null)
    {
        if (is_null($template)) {
            // if not defined: $template
            $action_path = $this->request->getActionPath();
            $template = strtolower($action_path) . '.php';
        }

        $template_dir = $this->moony->template_dir;
        if (strlen($template_dir) == 0 ||
            !file_exists($template_dir)) {
            // not defined: template_dir
            trigger_error('Not defined: template_dir', E_USER_ERROR);
            return false;
        }

        $template = $template_dir . DIRECTORY_SEPARATOR . $template;
        if (!file_exists($template)) {
            // not found
            return false;
        }

        if (is_array($local_vars)) {
            extract($local_vars, EXTR_SKIP);
        }

        ob_start();
        ob_implicit_flush(false);
        include $template;
        $result = ob_get_contents();
        ob_end_clean();

        return $result;
    }

    /**
     * テンプレートの処理結果を出力します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、
     * HTTPのステータスで404を送出して処理を終了します。
     * テンプレートからアクションクラスのpublicなメンバ変数には
     * <?php h($this->foo); ?>とアクセスすることができます。
     * $local_varsが設定されている場合、値は展開されて
     * <?php h($foo); ?>のように利用することができます。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function render(
        $template = null, $local_vars = null, $convert_encoding = true)
    {
        $result = $this->fetch($template, $local_vars);
        if ($result === false) {
            // not found
            ob_end_clean();
            $this->sendStatus(404);
            exit;
        }
        $this->renderText($result, $convert_encoding);
    }

    /**
     * 条件に合致した場合だけテンプレートの処理結果を出力します。
     * 処理の詳細はrenderメソッドを参照してください。
     * なお、テンプレート名を省略することはできません。
     *
     * @access public
     * @param bool $condition テンプレートの処理結果を出力する条件
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function renderIf($condition,
        $template, $local_vars = null, $convert_encoding = true)
    {
        if ($condition) {
            $this->render($template, $local_vars, $convert_encoding);
        }
    }

    /**
     * テキストを出力します。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     *
     * @access public
     * @param string $text 出力するテキスト
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function renderText($text, $convert_encoding = true)
    {
        if ($convert_encoding) {
            if (!is_null($this->moony->output_encoding)) {
                $text = mb_convert_encoding($text,
                    $this->moony->output_encoding,
                    $this->moony->internal_encoding);
            }
        }
        print($text);
    }

    /**
     * 値をJSON形式に変換して出力します。
     * このメソッドを正常に使用するためには、
     * JSONエクステンションまたはJsphonが利用可能である必要があります。
     * いずれもインストールされていない場合、警告を発生、falseを返します。
     * $mime_typeはHTTPヘッダのContent-Typeとして送出されます。
     *
     * @access public
     * @param mixed $value JSONに変換して出力する値
     * @param string $mime_type 出力時に使用されるMIMEタイプ
     * @see http://pecl.php.net/package/json
     * @see http://www.hawklab.jp/jsonencoder/
     * @see http://www.rfc-editor.org/rfc/rfc4627.txt
     */
    function renderJson($value, $mime_type = 'application/json')
    {
        // to UTF-8
        if ($this->moony->internal_encoding != 'UTF-8') {
            mb_convert_variables(
                'UTF-8', $this->moony->internal_encoding, $value);
        }

        $json = null;
        if (function_exists('json_encode')) {
            // use JSON extension
            $json = json_encode($value);
        } else if (class_exists('Jsphon')) {
            // use Jsphon
            $json = Jsphon::encode($value);
        } else {
            trigger_error(
                'Not loaded: JSON extension or Jsphon', E_USER_WARNING);
            return false;
        }

        if (!is_null($mime_type)) {
            header("Content-Type: {$mime_type}");
        }
        header('Content-Length: ' . strlen($json));
        print($json);
    }

    /**
     * リダイレクト処理を行い、処理を終了します。
     * リダイレクト先が「[a-z]://」で始まる場合、
     * 対象のURLにリダイレクト処理を行います。
     * そうでない場合、リダイレクト先をアクション名とみなして
     * 該当のアクションにリダイレクト処理を行います。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     * $action_argsが設定されていて、アクションにリダイレクトする場合、
     * URLの末尾にPATH_INFOとして付加、パラメータとして渡します。
     *
     * @access public
     * @param string $to リダイレクト先（URLまたはアクション名）
     * @param int $status ステータスコード（初期値は302）
     * @param array $action_args アクションに渡すパラメータの配列
     */
    function redirect($to, $status = 302, $action_args = null)
    {
        $url = null;
        if (preg_match('#^[a-z]+\://#', $to)) {
            // URL
            $url = $to;
        } else {
            // to the specific action
            if (!preg_match('/^[a-zA-Z0-9_]+$/', $to)) {
                trigger_error("Invalid action name: {$to}", E_USER_ERROR);
                return false;
            }
            $action_path = strtolower(strtr($to, '_', '/'));
            $url = $this->request->getHostUrl()
                 . $this->request->getActionBase()
                 . '/' . $action_path;
            if (preg_match('/index$/', $url)) {
                // remove 'index'
                $url = preg_replace('/index$/', '', $url);
            }
            if (is_array($action_args)) {
                $url = rtrim($url, '/');
                foreach ($action_args as $action_arg) {
                    $url .= '/' . urlencode($action_arg);
                }
            }
        }
        if (ob_get_length() !== false) {
            ob_end_clean();
        }
        if (!is_null($status)) {
            $this->sendStatus($status);
        }
        header("Location: {$url}");
        echo '<html><head><meta http-equiv="refresh" content="0;url='
            . htmlentities($url, ENT_QUOTES) . '"></head></html>';
        exit;
    }

    /**
     * 条件に合致した場合だけリダイレクト処理を行います。
     * 処理の詳細はredirectメソッドを参照してください。
     *
     * @access public
     * @param bool $condition リダイレクト処理を行う条件
     * @param string $to リダイレクト先（URLまたはアクション名）
     * @param int $status ステータスコード（初期値は302）
     * @param array $action_args アクションに渡すパラメータの配列
     */
    function redirectIf($condition, $to, $status = 302, $action_args = null)
    {
        if ($condition) {
            $this->redirect($to, $status, $action_args);
        }
    }

    /**
     * 検証結果を取得します。
     * 検証エラーの有無を返します。
     * エラーメッセージが設定されていればその配列を返します。
     *
     * @access public
     * @return bool|array 検証エラーの有無、エラーメッセージの配列
     */
    function errors() {
        $has_error = Moony_Registry::get('_moony_has_error');
        if ($has_error === true) {
            $errors = Moony_Registry::get('_moony_errors');
            if (is_array($errors) && count($errors) > 0) {
                return $errors;
            }
            return true;
        }
        return false;
    }

    /**
     * HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param int $status ステータスコード
     * @see http://www.rfc-editor.org/rfc/rfc1945.txt
     * @see http://www.rfc-editor.org/rfc/rfc2616.txt
     */
    function sendStatus($status)
    {
        // HTTP/1.1?
        $http_version = $this->request->getHttpVersion();
        $is_http_11 = $http_version == '1.1';
        // messages
        $messages = array(
            // Informational 1xx
            100 => 'Continue',
            101 => 'Switching Protocols',
            // Success 2xx
            200 => 'OK',
            201 => 'Created',
            202 => 'Accepted',
            203 => 'Non-Authoritative Information',
            204 => 'No Content',
            205 => 'Reset Content',
            206 => 'Partial Content',
            // Redirection 3xx
            300 => 'Multiple Choices',
            301 => 'Moved Permanently',
            302 => 'Found',  // 1.1
            303 => 'See Other',
            304 => 'Not Modified',
            305 => 'Use Proxy',
            // 306 is no longer used but still reserved
            307 => 'Temporary Redirect',
            // Client Error 4xx
            400 => 'Bad Request',
            401 => 'Unauthorized',
            402 => 'Payment Required',
            403 => 'Forbidden',
            404 => 'Not Found',
            405 => 'Method Not Allowed',
            406 => 'Not Acceptable',
            407 => 'Proxy Authentication Required',
            408 => 'Request Timeout',
            409 => 'Conflict',
            410 => 'Gone',
            411 => 'Length Required',
            412 => 'Precondition Failed',
            413 => 'Request Entity Too Large',
            414 => 'Request-URI Too Long',
            415 => 'Unsupported Media Type',
            416 => 'Requested Range Not Satisfiable',
            417 => 'Expectation Failed',
            // Server Error 5xx
            500 => 'Internal Server Error',
            501 => 'Not Implemented',
            502 => 'Bad Gateway',
            503 => 'Service Unavailable',
            504 => 'Gateway Timeout',
            505 => 'HTTP Version Not Supported',
            509 => 'Bandwidth Limit Exceeded'
        );
        if (isset($messages[$status])) {
            if (!$is_http_11) {
                $messages[302] = 'Moved Temporarily';
            }
            header("HTTP/{$http_version} {$status} {$messages[$status]}");
        }
    }

    /**
     * 任意のファイルをダウンロードさせます。
     * $nameが設定されていない場合、元のファイル名を使用します。
     * 指定されたファイルが存在しないか、
     * ファイルを読み込むことができない場合にfalseを返します。
     * 使用例: $this->sendFile('/path/to/foo.zip');
     *
     * @access public
     * @param string $path ダウンロードするファイルのパス
     * @param string $name ダウンロードするファイルにつける名前
     * @param int $buffer_size ファイル読み込み時のバッファサイズ
     * @return null|bool ファイルが存在しない場合false
     */
    function sendFile($path, $name = null, $buffer_size = 4096)
    {
        $path = realpath($path);
        if ($path === false || !is_file($path)) {
            return false;
        }
        if (is_null($name)) {
            // original name
            $name = basename($path);
        }

        $fp = fopen($path, 'rb');
        if ($fp === false) {
            return false;
        }

        header('Content-Type: application/octet-stream');
        header("Content-Disposition: attachment; filename=\"{$name}\"");
        header('Content-Length: ' . filesize($path));

        while (!feof($fp)) {
            echo fread($fp, $buffer_size);
        }
        fclose($fp);
    }

    /**
     * ページをキャッシュさせないようにします。
     *
     * @access public
     */
    function disableCache()
    {
        header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
        header('Cache-Control: no-store, no-cache, must-revalidate');
        header('Cache-Control: post-check=0, pre-check=0', false);
        header('Pragma: no-cache');
    }

    /**
     * アップロードされたファイルの保存を行います。
     * 保存先のディレクトリは前もって用意されている必要があります。
     * なお、配列を用いた複数ファイルのアップロードには対応していません。
     *
     * @access public
     * @param string $name アップロードファイル名
     * @param string $path 保存先のパス
     * @param int $mode 保存先のファイルのモード
     * @param bool $delete 保存後に一時ファイルを削除するかどうか
     * @return bool アップロードに成功したかどうか
     */
    function saveFile($name, $path, $mode = 0666, $delete = false)
    {
        $file_info = $this->request->getFile($name);
        if (is_null($file_info) ||
            !is_uploaded_file($file_info['tmp_name'])) {
            // not uploaded
            return false;
        }
        $dir = dirname($path);
        if (!file_exists($dir) || !is_dir($dir)) {
            // directory not found
            return false;
        }
        if ($file_info['error'] != UPLOAD_ERR_OK) {
            // upload error
            return false;
        }
        if (move_uploaded_file($file_info['tmp_name'], $path)) {
            // move uploaded file
            chmod($path, $mode);
            if ($delete) {
                unlink($file_info['tmp_name']);
            }
            return true;
        }
        return false;
    }

    /**
     * ワンタイムトークンのチェックを行います。
     * リクエストパラメータとして渡された値と、
     * セッションに保存されていた値が一致する場合のみ妥当と判断します。
     *
     * @access public
     * @return bool 妥当なワンタイムトークンかどうか
     */
    function checkToken()
    {
        // remove from the session
        $saved = $this->session->remove('_moony_token');
        $received = $this->request->get('_moony_token');
        if (strlen($saved) > 0 && strlen($received) > 0) {
            return $saved === $received;
        }
        return false;
    }

    /**
     * フラッシュ変数を設定します。
     * フラッシュ変数はセッション内に一時的に保存され、
     * 次のアクションが実行されるタイミングで
     * 値の取り出しおよび破棄が行われます。
     * 例えば、現在のアクションで $this->flash('foo', 'bar'); した場合、
     * 次のアクションのプロパティ'foo'として'bar'が設定されます。
     * （リクエストパラメータと名前が衝突する場合、
     * フラッシュ変数の値が優先されるので注意してください。）
     * セッションが開始されていない場合、falseを返します。
     *
     * @access public
     * @param string $name フラッシュ変数の名称
     * @param mixed $value フラッシュ変数の値
     */
    function flash($name, $value)
    {
        if (!$this->session->isStarted()) {
            return false;
        }
        $flash_vars = $this->session->get('_moony_flash');
        if (!is_array($flash_vars)) {
            $flash_vars = array();
        }
        $flash_vars[$name] = $value;
        $this->session->set('_moony_flash', $flash_vars);
    }
}
?>
