<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

require_once 'Console/Getopt.php';
require_once dirname(dirname(__FILE__)) . '/Moony.php';
define('DS', DIRECTORY_SEPARATOR);

$cg =& new Console_Getopt();
$args = $cg->readPHPArgv();
$results = $cg->getopt($args, 'd:ehv',
    array('dir=', 'entrypoint', 'help', 'version'));
if (PEAR::isError($results)) {
    echo '  [error] ' . $results->getMessage() . "\n";
    exit;
}

$opts = $results[0];
$action_names = $results[1];

$base_dir = getcwd();
$create_entry_point = false;
$show_help = false;
$show_version = false;

foreach ($opts as $opt) {
    switch ($opt[0]) {
        case 'd':
        case '--dir':
            // ベースディレクトリ指定
            $base_dir = $opt[1];
            break;
        case 'e':
        case '--entrypoint':
            // エントリポイントを生成
            $create_entry_point = true;
            break;
        case 'h':
        case '--help':
            $show_help = true;
            break;
        case 'v':
        case '--version':
            $show_version = true;
            break;
    }
}

// インスタンス生成
$command =& new Moony_Command($base_dir);

if ($show_version) {
    // バージョンを表示
    $command->showVersion();
    $show_help = false;
}

if ($create_entry_point) {
    // エントリポイント生成
    $command->createEntryPoint();
    $show_help = false;
}

foreach ($action_names as $action_name) {
    // アクション生成
    $command->createAction($action_name);
    $show_help = false;
}

if ($show_help) {
    // ヘルプを表示して終了
    $command->showHelp();
    exit;
}

/**
 * コマンドラインから呼び出されるクラスです。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Command
{
    /**
     * 自動生成の基準となるディレクトリ
     * @var string
     */
    var $_base_dir;

    /**
     * アクションクラスを格納するディレクトリ
     * @var string
     */
    var $_action_dir;

    /**
     * テンプレートファイルを格納するディレクトリ
     * @var string
     */
    var $_template_dir;

    /**
     * コンストラクタです。
     *
     * @access public
     * @param string $base_dir 自動生成の基準となるディレクトリ
     */
    function Moony_Command($base_dir)
    {
        $this->_base_dir = realpath(rtrim($base_dir, DS));
        $this->_action_dir = $this->_base_dir . DS . 'actions';
        $this->_template_dir = $this->_base_dir . DS . 'templates';
    }

    /**
     * Moonyを実行するエントリポイントを生成します。
     * また、同時に.htaccessファイルの生成も行います。
     *
     * @access public
     */
    function createEntryPoint()
    {
        $entry_point = $this->_base_dir . DS . 'index.php';
        $this->_createFile($entry_point, $this->_getEntryPointSkeleton());
        $htaccess = $this->_base_dir . DS . '.htaccess';
        $this->_createFile($htaccess, $this->_getHtaccess());
    }

    /**
     * アクションクラスとテンプレートファイルを生成します。
     *
     * @access public
     * @param string $action_name アクション名称
     */
    function createAction($action_name)
    {
        $action_path = implode(DS,
            array_map('ucfirst', explode('_', $action_name)));
        $action = $this->_action_dir . DS . $action_path . '.php';
        $template =
            $this->_template_dir . DS . strtolower($action_path) . '.php';
        $action_skl = $this->_getActionClassSkeleton($action_name);
        $template_skl =
            $this->_getTemplateSkeleton($action_name, $action, $template);
        $this->_createFile($action, $action_skl);
        $this->_createFile($template, $template_skl);
    }

    /**
     * ヘルプメッセージを表示します。
     *
     * @access public
     */
    function showHelp()
    {
        echo "\n";
        echo "  Usage: moony [options] <action_name> ...\n";
        echo "\n";
        echo "  'Foo' generates:\n";
        echo "      - actions/Foo.php ......... the action class\n";
        echo "      - templates/foo.php ....... the template file\n";
        echo "  'Foo_Bar' generates:\n";
        echo "      - actions/Foo/Bar.php ..... the action class\n";
        echo "      - templates/foo/bar.php ... the template file\n";
        echo "\n";
        echo "  More than one action may be specified at once.\n";
        echo "\n";
        echo "  Options:\n";
        echo "    -d DIR, --dir=DIR\n";
        echo "        root directory used when generating files\n";
        echo "    -e, --entrypoint\n";
        echo "        generate an entry point (index.php)\n";
        echo "        and a '.htaccess' file\n";
        echo "    -h, --help\n";
        echo "        show this help\n";
        echo "    -v, --version\n";
        echo "        show the version of Moony\n";
        echo "\n";
    }

    /**
     * Moonyのバージョンを表示します。
     *
     * @access public
     */
    function showVersion()
    {
        $version = MOONY_VERSION . ' (beta)';
        echo " [version] {$version}\n";
    }

    /**
     * エントリポイントの雛形を取得します。
     *
     * @access protected
     * @return string エントリポイントの雛形
     */
    function _getEntryPointSkeleton()
    {
        $skeleton = "<?php\n"
                  . "require_once 'Moony.php';\n"
                  . "\n"
                  . "\$moony = new Moony();\n"
                  . "\$moony->setActionDir('actions');\n"
                  . "\$moony->setTemplateDir('templates');\n"
                  . "// \$moony->setInputEncoding('SJIS');\n"
                  . "// \$moony->setOutputEncoding('SJIS');\n"
                  . "// \$moony->setInternalEncoding('UTF-8');\n"
                  . "// \$moony->setSessionOptions(array());\n"
                  . "// \$moony->setSessionAutoStart(true);\n"
                  . "\$moony->start();\n"
                  . "?>\n";
        return $skeleton;
    }

    /**
     * .htaccessファイルの雛形を取得します。
     *
     * @access protected
     * @return string .htaccessファイルの雛形
     */
    function _getHtaccess()
    {
        $skeleton = "<IfModule mod_rewrite.c>\n"
                  . "    RewriteEngine On\n"
                  . "    RewriteCond %{REQUEST_FILENAME} !-f\n"
                  . "    RewriteCond %{REQUEST_FILENAME} !-d\n"
                  . "    RewriteRule ^(.*)$ index.php [QSA,L]\n"
                  . "</IfModule>\n";
        return $skeleton;
    }

    /**
     * アクションクラスの雛形を取得します。
     *
     * @access protected
     * @param string $action_name アクション名
     * @return string アクションクラスの雛形
     */
    function _getActionClassSkeleton($action_name)
    {
        $skeleton = "<?php\n"
                  . "class {$action_name} extends Moony_Action\n"
                  . "{\n"
                  . "    function execute()\n"
                  . "    {\n"
                  . "        \$this->render();\n"
                  . "    }\n"
                  . "}\n"
                  . "?>\n";
        return $skeleton;
    }

    /**
     * テンプレートファイルの雛形を取得します。
     *
     * @access protected
     * @param string $action_name アクション名
     * @param string $action_class_file アクションクラス
     * @param string $template_file テンプレートファイル
     * @return string テンプレートファイルの雛形
     */
    function _getTemplateSkeleton(
        $action_name, $action_class_file, $template_file)
    {
        $skeleton = "<html>\n"
                  . "<head>\n"
                  . "    <title>It worked!</title>\n"
                  . "</head>\n"
                  . "<body>\n"
                  . "<h1>It worked!</h1>\n"
                  . "<p>Congratulations on your "
                  . "action page of Moony, '{$action_name}'.</p>\n"
                  . "<h2>working files</h2>\n"
                  . "<ul>\n"
                  . "    <li>action class: {$action_class_file}</li>\n"
                  . "    <li>template file: {$template_file}</li>\n"
                  . "</ul>\n"
                  . "</body>\n"
                  . "</html>\n";
        return $skeleton;
    }

    /**
     * ディレクトリを生成します。
     *
     * @access protected
     * @param string $dir ディレクトリ名称
     */
    function _createDir($dir)
    {
        if (file_exists($dir)) {
            echo "   [error] already exists: {$dir}\n";
            return;
        }
        $parent_dir = dirname($dir);
        if (!file_exists($parent_dir)) {
            $this->_createDir($parent_dir);
        }
        mkdir($dir);
        echo "  [create] {$dir}\n";
    }

    /**
     * プレーンテキストファイルを生成します。
     *
     * @access protected
     * @param string $file ファイル名称
     * @param string $content ファイルの内容
     */
    function _createFile($file, $content = null)
    {
        if (file_exists($file)) {
            echo "   [error] already exists: {$file}\n";
            return;
        }
        $dir = dirname($file);
        if (!file_exists($dir)) {
            $this->_createDir($dir);
        }
        $fp = fopen($file, 'w');
        if (!is_null($content)) {
            fwrite($fp, $content);
        }
        fclose($fp);
        echo "  [create] {$file}\n";
    }
}
?>
