<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

/**
 * リクエストパラメータ等のフィルタ処理を行います。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Filter
{
    /**
     * 内部エンコーディング
     * @var string
     */
    var $_internal_encoding;

    /**
     * コンストラクタです。
     *
     * @access public
     * @param string $internal_encoding 内部エンコーディング
     */
    function Moony_Filter($internal_encoding)
    {
        $this->_internal_encoding = $internal_encoding;
    }

    /**
     * 値の内容を半角英字のみにします。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function filterAlpha(&$var)
    {
        $this->filterPattern($var, '/[^[:alpha:]]/');
    }

    /**
     * 値の内容を半角英数字のみにします。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function filterAlnum(&$var)
    {
        $this->filterPattern($var, '/[^[:alnum:]]/');
    }

    /**
     * 値の内容を半角数字のみにします。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function filterDigit(&$var)
    {
        $this->filterPattern($var, '/[^\d]/');
    }

    /**
     * 値の内容を指定されたパターンに合致する文字のみにします。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function filterPattern(&$var, $pattern)
    {
        if (is_array($var)) {
            foreach ($var as $key => $value) {
                $var[$key] = preg_replace($pattern, '', $value);
            }
        } else {
            $var = preg_replace($pattern, '', $var);
        }
    }

    /**
     * 整数に変換します。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function toInt(&$var)
    {
        $var = $this->_apply($var, 'intval');
    }

    /**
     * 英字を小文字に変換します。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function toLowercase(&$var)
    {
        $var = $this->_apply($var, 'strtolower');
    }

    /**
     * 英字を大文字に変換します。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function toUppercase(&$var)
    {
        $var = $this->_apply($var, 'strtoupper');
    }

    /**
     * かなをひらがなに変換します。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function toHiragana(&$var)
    {
        $this->toKana($var, 'HVc');
    }

    /**
     * かなを全角カタカナに変換します。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function toKatakana(&$var)
    {
        $this->toKana($var, 'KVC');
    }

    /**
     * かなを変換します。
     * $optionにはmb_convert_kana関数と
     * 同様のオプションを指定できます。
     *
     * @access public
     * @param string|array $var 処理対象の値
     * @param string $option 変換オプション
     */
    function toKana(&$var, $option)
    {
        if (is_array($var)) {
            foreach ($var as $key => $value) {
                $var[$key] = mb_convert_kana(
                    $value, $option, $this->_internal_encoding);
            }
        } else {
            $var = mb_convert_kana(
                $var, $option, $this->_internal_encoding);
        }
    }

    /**
     * パスをファイル名のみにします。
     *
     * @access public
     * @param string|array $var 処理対象の値
     */
    function toBaseName(&$var)
    {
        $var = $this->_apply($var, 'basename');
    }

    /**
     * 文字をHTMLエンティティに変換します。
     * $quote_styleにはhtmlentities関数と
     * 同様の値を指定することができます。
     *
     * @access public
     * @param string|array $var 処理対象の値
     * @param string $quote_style クォーテーション文字の取り扱い方
     */
    function toHtmlEntities(&$var, $quote_style = ENT_COMPAT)
    {
        if (is_array($var)) {
            foreach ($var as $key => $value) {
                $var[$key] = htmlentities(
                    $value, $quote_style, $this->_internal_encoding);
            }
        } else {
            $var = htmlentities(
                $var, $quote_style, $this->_internal_encoding);
        }
    }

    /**
     * 文字列の両端にあるホワイトスペースを取り除きます。
     * $charlistにはtrim関数と同様の文字リストを指定できます。
     *
     * @acccess public
     * @param string|array $var 処理対象の値
     * @param string $charlist 除去する文字のリスト
     */
    function trim(&$var, $charlist = null)
    {
        if (is_null($charlist)) {
            $var = $this->_apply($var, 'trim');
        } else {
            if (is_array($var)) {
                foreach ($var as $key => $value) {
                    $var[$key] = trim($value, $charlist);
                }
            } else {
                $var = trim($var, $charlist);
            }
        }
    }

    /**
     * 文字列の左端にあるホワイトスペースを取り除きます。
     * $charlistにはltrim関数と同様の文字リストを指定できます。
     *
     * @acccess public
     * @param string|array $var 処理対象の値
     * @param string $charlist 除去する文字のリスト
     */
    function ltrim(&$var, $charlist = null)
    {
        if (is_null($charlist)) {
            $var = $this->_apply($var, 'ltrim');
        } else {
            if (is_array($var)) {
                foreach ($var as $key => $value) {
                    $var[$key] = ltrim($value, $charlist);
                }
            } else {
                $var = ltrim($var, $charlist);
            }
        }
    }

    /**
     * 文字列の右端にあるホワイトスペースを取り除きます。
     * $charlistにはrtrim関数と同様の文字リストを指定できます。
     *
     * @acccess public
     * @param string|array $var 処理対象の値
     * @param string $charlist 除去する文字のリスト
     */
    function rtrim(&$var, $charlist = null)
    {
        if (is_null($charlist)) {
            $var = $this->_apply($var, 'rtrim');
        } else {
            if (is_array($var)) {
                foreach ($var as $key => $value) {
                    $var[$key] = rtrim($value, $charlist);
                }
            } else {
                $var = rtrim($var, $charlist);
            }
        }
    }

    /**
     * 処理対象の値に指定されたコールバック関数を適用します。
     *
     * @access protected
     * @param string|array $var 処理対象の値
     * @param callback $callback 適用するコールバック関数
     * @return string|array 適用後の値
     */
    function _apply($var, $callback)
    {
        if (is_array($var)) {
            $var = array_map($callback, $var);
        } else {
            $var = call_user_func($callback, $var);
        }
        return $var;
    }
}
?>
