<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

/**
 * リクエスト情報を扱うクラスです。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Request
{
    /**
     * リクエストパラメータの連想配列
     * @var array
     */
    var $_params;

    /**
     * アップロードファイルに関する情報の連想配列
     * @var array
     */
    var $_files;

    /**
     * クッキーの値の連想配列
     * @var string
     */
    var $_cookie;

    /**
     * クエリーストリング
     * @var string
     */
    var $_query_string;

    /**
     * アクションパス
     * @var string
     */
    var $_action_path;

    /**
     * コンストラクタです。
     *
     * @access public
     */
    function Moony_Request()
    {
        $this->_params = $this->_filter($_POST + $_GET);
        $this->_files = $this->_filter($_FILES);
        $this->_cookie = $this->_filter($_COOKIE);

        // query string
        $query_string = $this->getServer('QUERY_STRING');
        if (!is_null($query_string)) {
            $this->_query_string = urldecode($query_string);
        }
    }

    /**
     * リクエストパラメータの値を取得します。
     * 該当する値が存在しない場合、$defaultを返します。
     *
     * @access public
     * @param string $name リクエストパラメータの名称
     * @param mixed $default 値が存在しない場合のデフォルト値
     * @return string|array リクエストパラメータの値
     */
    function get($name, $default = null)
    {
        if ($this->exists($name)) {
            return $this->_params[$name];
        }
        return $default;
    }

    /**
     * 全てのリクエストパラメータの値を連想配列として取得します。
     *
     * @access public
     * @return array 全てのリクエストパラメータの値の連想配列
     */
    function getAll()
    {
        return $this->_params;
    }

    /**
     * リクエストパラメータの存在有無を調べます。
     *
     * @access public
     * @param string $name リクエストパラメータの名称
     * @return bool リクエストパラメータの存在有無
     */
    function exists($name)
    {
        return isset($this->_params[$name]);
    }

    /**
     * アップロードファイルに関する情報を取得します。
     * $_FILESの1要素を取得します。
     *
     * @access public
     * @param string $name アップロードファイル名
     * @return array アップロードファイルに関する情報の連想配列
     */
    function getFile($name)
    {
        if (isset($this->_files[$name])) {
            return $this->_files[$name];
        }
    }

    /**
     * クッキーの値を取得します。$_COOKIEの1要素を取得します。
     *
     * @access public
     * @param string $name クッキーの名称
     * @return mixed クッキーに保存されていた値
     */
    function getCookie($name)
    {
        if (isset($this->_cookie[$name])) {
            return $this->_cookie[$name];
        }
    }

    /**
     * クエリーストリングを取得します。
     *
     * @access public
     * @return string クエリーストリング
     */
    function getQueryString()
    {
        return $this->_query_string;
    }

    /**
     * PATH_INFOを取得します。
     * 環境変数の「PATH_INFO」に依存しません。
     *
     * @access public
     * @return string PATH_INFO
     */
    function getPathInfo()
    {
        $request_uri = $this->getServer('REQUEST_URI');
        $script_name = $this->getServer('SCRIPT_NAME');

        $trim_pattern = '';
        if (preg_match('/^'. preg_quote($script_name, '/') . '/', $request_uri)) {
            // without mod_rewrite
            $trim_pattern = preg_quote($script_name, '/');
        } else {
            // with mod_rewrite
            $trim_pattern = preg_quote(dirname($script_name), '/');
        }

        $path_info = preg_replace("/^{$trim_pattern}/", '', $request_uri);
        if (strlen($this->_query_string) > 0) {
            $path_info = preg_replace('/' . preg_quote($this->_query_string, '/') . '$/', '', $path_info);
            $path_info = substr($path_info, 0, -1);
        }

        return $path_info;
    }

    /**
     * サーバのURLを取得します。
     * リクエストURLが「http://example.com/test/index.php/foo」の場合、
     * 「http://example.com」を返します。
     *
     * @access public
     * @return string サーバのURL
     */
    function getHostUrl()
    {
        $is_secure = $this->isSecure();
        $host_url = $is_secure ? 'https://' : 'http://';

        static $host_name_keys = array('HTTP_X_FORWARDED_HOST',
            'HTTP_X_FORWARDED_SERVER', 'HTTP_HOST', 'SERVER_NAME');
        static $port_keys = array('HTTP_X_FORWARDED_PORT', 'SERVER_PORT');

        // host name
        $host_name = '';
        foreach ($host_name_keys as $key) {
            $host_name = $this->getServer($key);
            if (!is_null($host_name)) {
                break;
            }
        }
        if (($delim_pos = strpos($host_name, ':')) !== false) {
            // remove port
            $host_name = substr($host_name, 0, $delim_pos);
        }

        // port
        $port = null;
        foreach ($port_keys as $key) {
            $port = $this->getServer($key);
            if (!is_null($port)) {
                if ($is_secure && $port == 443) {
                    $port = null;
                } else if (!$is_secure && $port == 80) {
                    $port = null;
                }
                break;
            }
        }

        $host_url .= strtolower($host_name);
        if (!is_null($port)) {
            // append port number
            $host_url .= ':' . $port;
        }

        return $host_url;
    }

    /**
     * 環境変数を取得します。
     * null文字が含まれている場合、除去します。
     *
     * @access public
     * @param string $name 環境変数の名称
     * @return string 環境変数の値
     */
    function getServer($name)
    {
        if (isset($_SERVER[$name])) {
            return str_replace("\0", '', $_SERVER[$name]);
        }
    }

    /**
     * 環境変数に格納されているリクエストヘッダを取得します。
     *
     * @access public
     * @param string $name リクエストヘッダの名称
     * @return string リクエストヘッダの値
     */
    function getHeader($name)
    {
        $server_name = 'HTTP_' . strtr(strtoupper($name), '-', '_');
        return $this->getServer($server_name);
    }

    /**
     * HTTPプロトコルのバージョンを取得します。
     *
     * @access public
     * @return string HTTPプロトコルのバージョン
     */
    function getHttpVersion()
    {
        $protocol = $this->getServer('SERVER_PROTOCOL');
        return (strpos($protocol, '1.1') !== false) ? '1.1' : '1.0';
    }

    /**
     * リクエストメソッドがPOSTかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがPOSTかどうか
     */
    function isPost()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'post';
    }

    /**
     * リクエストメソッドがGETかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがGETかどうか
     */
    function isGet()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'get';
    }

    /**
     * リクエストメソッドがPUTかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがPUTかどうか
     */
    function isPut()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'put';
    }

    /**
     * リクエストメソッドがDELETEかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがDELETEかどうか
     */
    function isDelete()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'delete';
    }

    /**
     * XMLHttpRequestを用いたアクセスかどうかを調べます。
     * Prototype.jsが付加する"X-Requested-With"ヘッダを利用して判定します。
     *
     * @access public
     * @return bool XMLHttpRequestを用いたアクセスかどうか
     */
    function isXmlHttpRequest()
    {
        $requested_with = $this->getHeader('X-Requested-With');
        return $requested_with == 'XMLHttpRequest';
    }

    /**
     * isXmlHttpRequestメソッドのエイリアスです。
     *
     * @access public
     * @return bool XMLHttpRequestを用いたアクセスかどうか
     */
    function isXhr()
    {
        return $this->isXmlHttpRequest();
    }

    /**
     * SSLを用いた通信かどうかを調べます。
     *
     * @access public
     * @return bool SSLを用いた通信かどうか
     */
    function isSecure()
    {
        return strtolower($this->getServer('HTTPS')) === 'on';
    }

    /**
     * アクションパスを設定します。
     *
     * @access public
     * @param string $action_path アクションパス
     */
    function setActionPath($action_path)
    {
        $this->_action_path = $action_path;
    }

    /**
     * アクションパスを取得します。
     *
     * @access public
     * @return string アクションパス
     */
    function getActionPath()
    {
        return $this->_action_path;
    }

    /**
     * アクションのURLの基準となるURLを取得します。
     * サーバがAapcheの場合、$_SERVER['REQUEST_URI']の末尾から
     * QUERY_STRINGとPATH_INFOを除去した値を返します。
     * サーバがIISの場合、$_SERVER['URL']の末尾から
     * PATH_INFOを除去した値を返します。
     * リクエストURLが「http://example.com/test/index.php/foo」の場合、
     * 「/test/index.php」を返します。mod_rewriteが使用されていて
     * 「http://example.com/test/foo」となっている場合、「/test」を返します。
     * このメソッドはMoony_Action::redirectAction()で使用されます。
     *
     * @access public
     * @return string アクションの基準となるURL
     */
    function getActionBase()
    {
        $request_uri = null;
        if (isset($_SERVER['REQUEST_URI'])) {
            // Apache
            $request_uri = $this->getServer('REQUEST_URI');
            if (($pos = strpos($request_uri, '?')) !== false) {
                $request_uri = substr($request_uri, 0, $pos);
            }
        } else {
            // IIS
            $request_uri = $this->getServer('URL');
        }
        $path_info = $this->getPathInfo();
        return substr($request_uri, 0,
            strlen($request_uri) - strlen($path_info));
    }

    /**
     * エンコーディングの変換を行います。
     *
     * @access public
     * @param string $from_encoding 変換元エンコーディング
     * @param string $to_encoding 変換先エンコーディング
     */
    function convertEncoding($from_encoding, $to_encoding)
    {
        mb_convert_variables($to_encoding, $from_encoding,
            $this->_params, $this->_files, $this->_query_string);
    }

    /**
     * リクエストパラメータのフィルタ処理を行います。
     * magic_quotes_gpcがONの場合、不要なスラッシュを除去します。
     * また、null文字が含まれている場合、除去します。
     *
     * @access protected
     * @param string|array $var フィルタ対象の値
     * @return string|array フィルタ済みの値
     */
    function _filter($var)
    {
        static $magic_quotes_gpc;
        if (is_null($magic_quotes_gpc)) {
            $magic_quotes_gpc = get_magic_quotes_gpc();
        }
        if (is_array($var)) {
            return array_map(array($this, '_filter'), $var);
        }
        $var = str_replace("\0", '', $var);
        if ($magic_quotes_gpc) {
            $var = stripslashes($var);
        }
        return $var;
    }
}
?>
